'use strict';

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _map2 = require('lodash/map');

var _map3 = _interopRequireDefault(_map2);

var _isBoolean2 = require('lodash/isBoolean');

var _isBoolean3 = _interopRequireDefault(_isBoolean2);

var _isRegExp2 = require('lodash/isRegExp');

var _isRegExp3 = _interopRequireDefault(_isRegExp2);

var _has2 = require('lodash/has');

var _has3 = _interopRequireDefault(_has2);

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _crypto = require('crypto');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isMemoryFileSystem = undefined;

/**
 * When 'webpack' program is used, constructor name is equal to 'NodeOutputFileSystem'.
 * When 'webpack-dev-server' program is used, constructor name is equal to 'MemoryFileSystem'.
 *
 * @param {Object} outputFileSystem
 * @returns {boolean}
 */
isMemoryFileSystem = function isMemoryFileSystem(outputFileSystem) {
    return outputFileSystem.constructor.name === 'MemoryFileSystem';
};

/**
 * @typedef {Object} options
 * @property {RegExp} test A regular expression used to test if file should be written. When not present, all bundle will be written.
 * @property {boolean} useHashIndex Use hash index to write only files that have changed since the last iteration (default: true).
 */

/**
 * @param {options} options
 * @returns {Object}
 */

exports.default = function () {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    var apply = undefined,
        hashIndex = undefined;

    if ((0, _has3.default)(options, 'test') && !(0, _isRegExp3.default)(options.test)) {
        throw new Error('options.test value must be an instance of RegExp.');
    } else {
        options.test = null;
    }

    if ((0, _has3.default)(options, 'useHashIndex') && !(0, _isBoolean3.default)(options.useHashIndex)) {
        throw new Error('options.useHashIndex value must be of boolean type.');
    } else {
        options.useHashIndex = true;
    }

    hashIndex = {};

    apply = function apply(compiler) {
        compiler.plugin('done', function (stats) {
            var files = undefined,
                outputPath = undefined;

            if (!isMemoryFileSystem(compiler.outputFileSystem)) {
                return;
            }

            if (stats.compilation.errors.length) {
                return;
            }

            if (!compiler.options.devServer.outputPath) {
                throw new Error('devServer.outputPath is not defined.');
            }

            outputPath = compiler.options.devServer.outputPath;

            files = (0, _map3.default)(stats.compilation.assets, 'existsAt');

            (0, _forEach3.default)(files, function (relativeAssetPath) {
                var assetAbsolutePath = undefined,
                    assetBody = undefined,
                    assetBodyHash = undefined,
                    outputFilePath = undefined;

                if (options.test && !options.test.test(relativeAssetPath)) {
                    return;
                }

                assetAbsolutePath = _path2.default.join(compiler.options.output.path, relativeAssetPath);
                assetBody = compiler.outputFileSystem.readFileSync(assetAbsolutePath, 'utf8');

                if (options.useHashIndex) {
                    assetBodyHash = (0, _crypto.createHash)('sha256').update(assetBody).digest('hex');

                    if (hashIndex[relativeAssetPath] && hashIndex[relativeAssetPath] === assetBodyHash) {
                        return;
                    }

                    hashIndex[relativeAssetPath] = assetBodyHash;
                }

                outputFilePath = _path2.default.join(outputPath, relativeAssetPath);

                _mkdirp2.default.sync(_path2.default.dirname(outputFilePath));

                _fs2.default.writeFileSync(outputFilePath, assetBody);
            });
        });
    };

    return {
        apply: apply
    };
};

module.exports = exports['default'];
//# sourceMappingURL=index.js.map
